/*
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MM          MM
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MMM        MMM
 *          TT        HH          HH  OO          OO  MMMM      MMMM 
 *          TT        HH          HH  OO          OO  MM MM    MM MM
 *          TT        HH          HH  OO          OO  MM  MM  MM  MM 
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM   MMMM   MM
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM    MM    MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM 
 *
 *                      l'mulateur Thomson TO7-70
 *
 *  Copyright (C) 1996 Sylvain Huet, 1999-2002 Eric Botcazou.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 *  Module     : dos/graphic.c
 *  Version    : 1.5.5
 *  Cr par   : Eric Botcazou 1999
 *  Modifi par: Eric Botcazou 03/05/2002
 *
 *  Gestion de l'affichage du TO7-70.
 */


#ifndef SCAN_DEPEND
   #include <stdio.h>
   #include <pc.h>
   #include <go32.h>
   #include <allegro.h>
#endif

#include "alleg/graphic.h"
#include "alleg/main.h"
#include "to7.h"


#define BLIT_GPL(src, addr)  __asm__ __volatile__ (                     \
                                 "movw %w0, %%fs\n"                     \
                                 "roll $3, %%eax\n"                     \
                                 "addl $0xa0000, %%eax\n"               \
                                 "movl (%%ebx), %%edx\n"                \
                                 "movl %%edx, %%fs: (%%eax)\n"          \
                                 "movl 4(%%ebx), %%edx\n"               \
                                 "movl %%edx, %%fs: 4(%%eax)\n"         \
                                 :                                      \
                                 : "o" (vga_sel), "b" (src), "a" (addr) \
                                 : "edx"                                \
                             )


int need_border_refresh;

#define COLOR_SHIFT  17
static PALETTE palette={ { 0, 0, 0},  /* bordure de l'cran */
                         { 0, 0, 0},  /*     */
                         {60,63,63},  /*     */
                         { 0, 0, 0},  /* GUI */
                         {47,50,50},  /*     */
                         {60,63,63},  /*     */
                         { 0,63, 0}   /* LED verte */
};

static int border_color;
static int graphic_mode;
static unsigned short vga_sel;
static BITMAP *screen_buffer;



/* RefreshBorder:
 *  Change la couleur du pourtour de l'cran.
 */
void RefreshBorder(void)
{
    set_color(0, &palette[border_color]);
    need_border_refresh=FALSE;
}



/* SetBorderColor:
 *  Fixe la couleur du pourtour de l'cran en copiant
 *  le triplet RGB choisi sur la couleur 0.
 */
static void SetBorderColor(int color)
{
    color += COLOR_SHIFT;

    if (border_color != color)
    {
        border_color=color;
        need_border_refresh=TRUE;
    }
}



/* DrawGPL:
 *  Affiche un Groupe Point Ligne (un octet de VRAM).
 */
static void DrawGPL(int addr, int pt, int col)
{
    register int i;
    unsigned int c1,c2;
    char *buffer_p, *buffer_psrc;

    buffer_psrc = buffer_p = screen_buffer->line[addr/40]+(addr%40)*8;

    c1=((col>>3)&7)+(((~col)&0x40)>>3)+COLOR_SHIFT;
    c2=(col&7)+(((~col)&0x80)>>4)+COLOR_SHIFT;

    for (i=0; i<8; i++)
        *buffer_p++ = (0x80>>i)&pt ? c1 : c2;

    if (graphic_mode)
        BLIT_GPL(buffer_psrc, addr);
}

END_OF_FUNCTION(DrawGPL);



/* RetraceScreen:
 *  Rafrachit une portion de l'cran du TO7-70.
 */
void RetraceScreen(int x, int y, int width, int height)
{
    blit(screen_buffer, screen, x, y, x, y, width, height);
}



/* SetGraphicMode:
 *  Slectionne le mode graphique de l'mulateur.
 */
int SetGraphicMode(int mode)
{
    switch (mode)
    {
        case INIT:
            set_gfx_mode(GFX_VGA, 320, 200, 0, 0);
            set_palette(palette);
            graphic_mode=TRUE;
            break;

        case RESTORE:
            set_gfx_mode(GFX_VGA, 320, 200, 0, 0);
            set_palette(palette);
            blit(screen_buffer, screen, 0, 0, 0, 0, 320, 200);
            graphic_mode=TRUE;
            break;

        case SHUTDOWN:
            get_palette(palette); /* on sauvegarde la palette */
            set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
            graphic_mode=FALSE;
            break;

    } /* end of switch */

    return TRUE;
}



/* SetDiskLed:
 *  Allume/teint la LED du lecteur de disquettes.
 */
static void SetDiskLed(int led_on)
{
    if (graphic_mode)
    {
        if (led_on)
        {
            rect(screen,314,0,319,5,1);
            rectfill(screen,315,1,318,4,6);
	}
	else
	    blit(screen_buffer,screen,314,0,314,0,6,6);
    }
}



/* InitGraphic:
 *  Initialise le pilote graphique.
 */
int InitGraphic(int gfx_mode)
{
    int i;

    if (gfx_mode != GFX_MODE40)
       return FALSE;

    set_color_depth(8);

    /* rcupration de la palette du TO7-70 */
    for (i=0; i<TO7_NCOLORS; i++)
    {
        int red, green, blue;

        to7_QueryColor(i, &red, &green, &blue);

        palette[i+COLOR_SHIFT].r = red >> 2;
        palette[i+COLOR_SHIFT].g = green >> 2;
        palette[i+COLOR_SHIFT].b = blue >> 2;
    }

    /* pointeur de la souris de la GUI */
    palette[16] = palette[1];
    palette[255] = palette[2];

    SetGraphicMode(INIT);

    vga_sel = _dos_ds;
    screen_buffer = create_bitmap(320, 200);

    to7_DrawGPL = DrawGPL;
    to7_SetBorderColor = SetBorderColor;
    to7_SetDiskLed = SetDiskLed;

    LOCK_VARIABLE(graphic_mode);
    LOCK_FUNCTION(DrawGPL);

    return TRUE;
}



/* Screenshot:
 *  Prend une photo d'cran de l'mulateur.
 */
void Screenshot(void)
{
    static int count;
    char filename[FILENAME_LENGTH];
    BITMAP *bmp;
    PALETTE palette;

    get_palette(palette);
    bmp = create_sub_bitmap(screen, 0, 0, SCREEN_W, SCREEN_H);
    sprintf(filename, "thom_%02d.bmp", count++);
    save_bmp(filename, bmp, palette);
    destroy_bitmap(bmp);
}

