/*
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MM          MM
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MMM        MMM
 *          TT        HH          HH  OO          OO  MMMM      MMMM 
 *          TT        HH          HH  OO          OO  MM MM    MM MM
 *          TT        HH          HH  OO          OO  MM  MM  MM  MM 
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM   MMMM   MM
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM    MM    MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM 
 *
 *                      l'mulateur Thomson TO7-70
 *
 *  Copyright (C) 1996 Sylvain Huet, 1999-2002 Eric Botcazou.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 *  Module     : dos/keybint.c
 *  Version    : 1.5.5
 *  Cr par   : Eric Botcazou octobre 1999
 *  Modifi par: Eric Botcazou 19/12/2000
 *
 *  Interface de gestion du clavier.
 */


#ifndef SCAN_DEPEND
   #include <allegro.h>
#endif

#include "alleg/main.h"
#include "to7.h"



/* KeyboardHandler:
 *  Handler des vnements bas-niveau clavier.
 */
static void KeyboardHandler(int key)
{
    int release=key&0x80;

    key&=0x7f;

    switch (key)
    {
        case KEY_ESC:
            if (!release)
                thom.command=CONTROL_PANEL;
            break;

        case KEY_F11:
            if (!release)
                thom.command=SCREENSHOT;
            break;

        case KEY_F12:
            if (!release)
                thom.command=DEBUGGER;
            break;

        default:
            to7_HandleKeyPress(key, release);
    }
}

static END_OF_FUNCTION(KeyboardHandler)



/* SetNoCapsLed:
 *  Modifie l'tat des Leds du clavier physique.
 */
static void SetNoCapsLed(int state)
{
    int flags = 0;

    if (!state)
        flags |= KB_CAPSLOCK_FLAG;

    /* mise  jour des leds */
    set_leds(flags);
}



/* InstallKeybint:
 *  Installe le module de gestion bas-niveau du clavier.
 */
void InstallKeybint(void)
{
    static int first=1;
    int mask=0, value=0;
  
    if (first)
    {
        mask |= (TO7_CAPSLOCK_FLAG);

        if (key_shifts&KB_CAPSLOCK_FLAG)
            value |= TO7_CAPSLOCK_FLAG;

        /* on prend dfinitivement le contrle des leds */
        key_led_flag = FALSE;
        first=0;
    }
       
    mask |= (TO7_SHIFT_FLAG | TO7_CTRL_FLAG);

    if (key_shifts&KB_SHIFT_FLAG)
        value |= TO7_SHIFT_FLAG;

    if (key_shifts&KB_CTRL_FLAG)
        value |= TO7_CTRL_FLAG;

    to7_InputReset(mask, value);

    keyboard_lowlevel_callback=KeyboardHandler;
}



/* ShutDownKeybint:
 *  Dsinstalle le module de gestion bas-niveau du clavier.
 */
void ShutDownKeybint(void)
{
    keyboard_lowlevel_callback=NULL;
}



/* InitKeybint:
 *  Initialise le module de gestion bas-niveau clavier.
 */
void InitKeybint(void)
{
    to7_SetNoCapsLed=SetNoCapsLed;
    LOCK_FUNCTION(KeyboardHandler);
}

