/*
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MM          MM
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MMM        MMM
 *          TT        HH          HH  OO          OO  MMMM      MMMM 
 *          TT        HH          HH  OO          OO  MM MM    MM MM
 *          TT        HH          HH  OO          OO  MM  MM  MM  MM 
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM   MMMM   MM
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM    MM    MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM 
 *
 *                      l'mulateur Thomson TO7-70
 *
 *  Copyright (C) 1996 Sylvain Huet, 1999-2003 Eric Botcazou.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 *  Module     : win/main.c
 *  Version    : 1.5.5
 *  Cr par   : Eric Botcazou septembre 2000
 *  Modifi par: Eric Botcazou 27/12/2003
 *
 *  Boucle principale de l'mulateur.
 */


#ifndef SCAN_DEPEND
   #include <stdio.h>
   #include <stdlib.h>
   #include <string.h>
   #include <allegro.h>
   #include <winalleg.h>
#endif

#include "alleg/graphic.h"
#include "alleg/gui.h"
#include "alleg/keybint.h"
#include "alleg/main.h"
#include "alleg/mouse.h"
#include "alleg/sound.h"
#include "win/gui.h"
#include "to7.h"


struct EmuTO thom={
    TRUE,
    TRUE,
    NONE
};

int frame;                  /* compteur de frame vido */
static volatile int tick;   /* compteur du timer       */

static void Timer(void)
{
    tick++;
}



/* RetraceCallback:
 *  Fonction callback de retraage de l'cran aprs
 *  restauration de l'application.
 */
static void RetraceCallback(void)
{
    acquire_screen();
    RetraceScreen(0, 0, SCREEN_W, SCREEN_H);
    release_screen();
}



/* RunTO7:
 *  Boucle principale de l'mulateur.
 */
static void RunTO7(int windowed_mode)
{
    if (!thom.sound_enabled)
        install_int_ex(Timer, BPS_TO_TIMER(TO7_FRAME_FREQ));

    frame=1;
    to7_ColdReset();

    do  /* boucle principale de l'mulateur */
    {
        thom.command=NONE;
        tick=frame;

        /* installation des handlers clavier, souris et son */
        InstallKeybint();
        InstallPointer();

        if (thom.sound_enabled && thom.exact_speed)
            StartSound();

        do  /* boucle d'mulation */
        {
            to7_DoFrame();

            if (need_border_refresh)  /*  rafrachissement du pourtour */
                RefreshBorder();

            RefreshScreen();

            /* synchronisation sur frquence relle */
            if (thom.exact_speed)
            {
                if (thom.sound_enabled)
                    PlaySoundBuffer();
                else
                    while (frame==tick)
	   	        Sleep(0);
            }

            frame++;
        }
        while (thom.command==NONE);  /* fin de la boucle d'mulation */

        /* dsinstallation des handlers clavier, souris et son */
        if (thom.sound_enabled && thom.exact_speed)
            StopSound();

        ShutDownPointer();
        ShutDownKeybint();

        /* xcution des commandes */
        if (thom.command==CONTROL_PANEL)
        {
            if (windowed_mode)
                DisplayControlPanelWin();
            else
                ControlPanel();
        }

        if (thom.command==SCREENSHOT)
            Screenshot();

        if (thom.command==RESET)
            to7_Reset();

        if (thom.command==COLD_RESET)
            to7_ColdReset();
    }
    while (thom.command != QUIT);  /* fin de la boucle principale */

    if (!thom.sound_enabled)
        remove_int(Timer);
}



/* ExitMessage:
 *  Affiche un message de sortie et sort du programme.
 */
static void ExitMessage(const char msg[])
{
    allegro_exit(); /* pour viter une fentre DirectX zombie */
    MessageBox(NULL, msg, "Thom - Erreur", MB_OK | MB_ICONERROR);
    exit(EXIT_FAILURE);
}



/* WinMain:
 *  Point d'entre du programme appel par l'API Win32.
 */
int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrevInst, LPSTR lpCmdLine, int nCmdShow)
{
    char version_name[]="Thom "TO7_VERSION_STR" (Windows/DirectX)";
    char memo_name[FILENAME_LENGTH]="\0";
    int disk_ctrl = TO7_CD90_640;
    int argc=0;
    char *argv[16];
    int gfx_mode=NO_GFX;
    int windowed_mode=FALSE;
    int i;

    /* conversion de la ligne de commande Windows */
    prog_inst = hInst;
    prog_icon = LoadIcon(hInst, "thomson_ico");

    if (*lpCmdLine)
    {
        argv[argc++]=lpCmdLine++;

        while (*lpCmdLine)
            if (*lpCmdLine == ' ')
            {
                *lpCmdLine++ = '\0';
                argv[argc++]=lpCmdLine++;
            }
            else
                lpCmdLine++;
    }
    
    /* traitement des paramtres */
    for (i=0; i<argc; i++)
    {
        if (!strcmp(argv[i],"-m"))
	    strcpy(memo_name, argv[++i]);
        else if (!strcmp(argv[i],"-nodisk"))
            disk_ctrl = TO7_NO_DISK_CTRL;
        else if (!strcmp(argv[i],"-fast"))
            thom.exact_speed=FALSE;
        else if (!strcmp(argv[i],"-nosound"))
            thom.sound_enabled=FALSE;
        else if (!strcmp(argv[i],"-mode40"))
            gfx_mode=GFX_MODE40;
        else if (!strcmp(argv[i],"-window"))
            gfx_mode=GFX_WINDOW;
        else
            ExitMessage("Paramtre inconnu.");
    }

    /* slection du mode graphique */
    if (gfx_mode == NO_GFX)
        SelectGraphicMode(&gfx_mode, NULL);

    /* l'initialisation de l'interface clavier, qui utilise un appel GDI, doit avoir lieu
       avant celle du module clavier d'Allegro, bas sur DirectInput */
    InitKeybint();

    /* initialisation de la librairie Allegro */
    set_uformat(U_ASCII);  /* pour les accents Latin-1 */
    allegro_init();
    install_keyboard();
    install_timer();

    /* dcoration de la fentre */
    set_window_title("Thom - l'mulateur TO7-70");
    prog_win = win_get_window();
    SetClassLong(prog_win, GCL_HICON,   (LONG) prog_icon);
    SetClassLong(prog_win, GCL_HICONSM, (LONG) prog_icon);

    /* initialisation de l'mulateur */
    printf("Initialisation de l'mulateur...");

    if (to7_Init(disk_ctrl) == TO7_ERROR)
    {
        /* la ROM contrleur disk est peut-tre manquante? */
        disk_ctrl = TO7_NO_DISK_CTRL;

        if (to7_Init(disk_ctrl) == TO7_ERROR)
            ExitMessage(to7_error_msg);
        else
        {
            printf("%s\n", to7_error_msg);
            printf("Initialisation de l'mulateur sans support des disquettes... ");
        }
    }

    printf("ok\n");

    /* initialisation du son */
    InitSound(51200);  /* 51200 Hz car 25600 Hz provoque des irrgularits du timer */
    
    /* initialisation du mode graphique */
    switch (gfx_mode)
    {
        case GFX_MODE40:
            if (!InitGraphic(GFX_MODE40))
                ExitMessage("Mode graphique non support.");
            break;

        case GFX_WINDOW:
            if (!InitGraphic(GFX_WINDOW))
                ExitMessage("Mode graphique non support.");
            windowed_mode = TRUE;
            break;
    }
    
    /* initialisation de l'interface utilisateur Allegro */
    if (!windowed_mode)   
       InitGUI(version_name, disk_ctrl != TO7_NO_DISK_CTRL, FALSE);

    /* chargement de la cartouche */
    if (memo_name[0])
    {
        to7_LoadMemo7(memo_name);
    }
    else if (disk_ctrl == TO7_NO_DISK_CTRL)
    {
        to7_LoadMemo7(".\\memo7\\basic.m7");
    }
    else
    {
        if (to7_LoadMemo7(".\\memo7\\basic128.m7") == TO7_ERROR)
            to7_LoadMemo7(".\\memo7\\basic.m7");
    }

    /* installation de la fonction callback de retraage de l'cran ncessaire
       pour les modes fullscreen */
    set_display_switch_callback(SWITCH_IN, RetraceCallback);

    /* et c'est parti !!! */
    RunTO7(windowed_mode);

    /* dsinstallation du callback *avant* la sortie du mode graphique */
    remove_display_switch_callback(RetraceCallback);

    /* sortie du mode graphique */
    SetGraphicMode(SHUTDOWN);

    /* sortie de l'mulateur */
    return 0;
}

